import 'dart:async';
import 'dart:io';

import 'package:flutterx_http/src/adapter/adapter.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class XHttpClientAdapter extends XHttpClient {
  final Set<CancellationToken> _requests = {};

  XHttpClientAdapter() : super.impl();

  @override
  Future<XHttpResponse> intercept(XHttpRequest request, XHttpRequestOptions options, InterceptorChain _) {
    final completer = Completer<XHttpResponse>();
    final client = HttpClient(context: options.context)
      ..userAgent = null
      ..idleTimeout = Duration.zero
      ..connectionTimeout = options.connectionTimeOut;
    options.cancel?.onCancel((_, __) => client.close(force: true));
    client.openUrl(request.method, request.url).then((ioRequest) async {
      final body = request.body;
      ioRequest
        ..followRedirects = options.maxRedirects > 0
        ..maxRedirects = options.maxRedirects
        ..contentLength = body == null ? 0 : body.contentLength ?? -1;
      request.headers.forEach(ioRequest.headers.set);
      if (body != null) {
        final stream = body.asStream();
        final onProgress = options.sendProgress;
        await ioRequest.addStream(onProgress != null ? stream.listenProgress(body.contentLength, onProgress) : stream);
      }
      final ioResponse = await ioRequest.close();
      final headers = <String, String>{};
      ioResponse.headers.forEach((key, values) => headers[key] = values.join(','));
      final onProgress = options.receiveProgress;
      final contentLength = ioResponse.contentLength == -1 ? null : ioResponse.contentLength;
      completer.completeResponse(XHttpResponse(
          request: request,
          body: XHttpContent.fromStream(
              onProgress != null ? ioResponse.listenProgress(contentLength, onProgress) : ioResponse,
              contentLength: contentLength),
          statusCode: ioResponse.statusCode,
          statusMessage: ioResponse.reasonPhrase,
          headers: headers,
          isRedirect: ioResponse.isRedirect));
    }, onError: (error, stackTrace) => completer.completeError(XHttpException(message: error.toString()), stackTrace));
    return completer.future;
  }

  @override
  void close() {
    for (final request in _requests.toSet()) _requests.remove(request..cancel());
  }
}

extension _StreamExt on Stream<List<int>> {
  Stream<List<int>> listenProgress(int? total, RequestProgress onProgress) {
    var loaded = 0;
    return transform(StreamTransformer.fromHandlers(
        handleData: (data, sink) {
          sink.add(data);
          loaded += data.length;
          onProgress(loaded, total);
        },
        handleError: (error, stack, sink) => sink.addError(error, stack),
        handleDone: (sink) => sink.close()));
  }
}
