part of '../adapter/adapter.dart';

@immutable
class XHttpResponse {
  final XHttpRequest request;
  final XHttpContent body;
  final int statusCode;
  final String statusMessage;
  final HeadersMap headers;
  final bool isRedirect;

  bool get isSuccess => statusCode ~/ 100 == 2;

  MediaType get contentType =>
      headers[HttpHeaders.contentTypeHeader]?.let(MediaType.parse) ?? MediaType('application', 'octet-stream');

  Encoding get encoding => contentType.parameters['charset']?.let(Encoding.getByName) ?? utf8;

  const XHttpResponse({
    required this.request,
    required this.body,
    required this.statusCode,
    required this.statusMessage,
    required this.headers,
    required this.isRedirect,
  });

  factory XHttpResponse.create({
    required XHttpRequest request,
    required XHttpContent body,
    int statusCode = 200,
    String statusMessage = 'OK',
    HeadersMap headers = const {},
    bool isRedirect = false,
  }) =>
      XHttpResponse(
          request: request,
          body: body,
          statusCode: statusCode,
          statusMessage: statusMessage,
          headers: headers,
          isRedirect: isRedirect);

  XHttpResponse copy({
    XHttpRequest? request,
    XHttpContent? body,
    int? statusCode,
    String? statusMessage,
    HeadersMap? headers,
    bool? isRedirect,
  }) =>
      XHttpResponse(
        request: request ?? this.request,
        body: body ?? this.body,
        statusCode: statusCode ?? this.statusCode,
        statusMessage: statusMessage ?? this.statusMessage,
        headers: headers ?? this.headers,
        isRedirect: isRedirect ?? this.isRedirect,
      );

  @override
  bool operator ==(Object other) =>
      other is XHttpResponse &&
      other.request == request &&
      other.body == body &&
      other.statusCode == statusCode &&
      other.statusMessage == statusMessage &&
      other.headers == headers &&
      other.isRedirect == isRedirect;

  @override
  int get hashCode => hashValues(
      request, body, statusCode, statusMessage, hashList(headers.keys), hashList(headers.values), isRedirect);

  @override
  String toString() => 'Response(${request.method} ${request.url}): $statusCode $statusMessage';
}
