import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:flutterx_http/flutterx_http.dart';

@immutable
class Request extends IRequest {
  final String method;
  final Uri url;
  final QueryParameters params;
  final HeadersMap headers;
  final dynamic body;
  final Encoding? encoding;

  const Request({
    required this.method,
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  });

  const Request.head({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'HEAD',
        body = null,
        encoding = null;

  const Request.get({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'GET',
        body = null,
        encoding = null;

  const Request.post({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'POST';

  const Request.put({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'PUT';

  const Request.patch({
    required this.url,
    this.params = const {},
    this.headers = const {},
    this.body,
    this.encoding,
  }) : method = 'PATCH';

  const Request.delete({
    required this.url,
    this.params = const {},
    this.headers = const {},
  })  : method = 'DELETE',
        body = null,
        encoding = null;

  @override
  XHttpRequest finalize() {
    final headers = {HttpHeaders.contentTypeHeader: 'application/json', ...this.headers};
    return XHttpRequest(
        method: method,
        url: url.appendQueryParameters(params),
        headers: headers,
        body: _encodeBody(body, encoding ?? utf8, headers[HttpHeaders.contentTypeHeader]));
  }

  static XHttpContent? _encodeBody(dynamic body, Encoding encoding, String? contentType) {
    if (body == null || body is XHttpContent) return body;
    switch (contentType) {
      case 'application/json':
        if (body is! String) body = jsonEncode(body);
        break;
      case 'application/x-www-form-urlencoded':
        if (body is! String)
          body = (body as Map)
              .entries
              .map((entry) =>
                  '${Uri.encodeQueryComponent(entry.key, encoding: encoding)}=${Uri.encodeQueryComponent(entry.value, encoding: encoding)}')
              .join('&');
        break;
      default:
        if (body is! String) throw UnsupportedError('Failed to encode request body for contentType: $contentType');
    }
    return XHttpContent.fromBytes(Uint8List.fromList(encoding.encode(body)));
  }
}
