import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:flutterx_http/flutterx_http.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

extension ResponseFutureConverter on Future<XHttpResponse> {
  Future<Uint8List> asBytes() => then((response) => response.asBytes());

  Future<Stream<List<int>>> asStream() => then((response) => response.asStream());

  Future<String> asText() => then((response) => response.asText());

  Future<T> asJson<T>(FromJson<T> fromJson) => then((response) => response.asJson(fromJson));

  Future<T> asJsonObject<T>(FromJsonObject<T> fromJson) => then((response) => response.asJsonObject(fromJson));

  Future<List<T>> asJsonList<T>(FromJsonObject<T> fromJson) => then((response) => response.asJsonList(fromJson));

  Future<Map<String, T>> asJsonMap<T>(FromJsonObject<T> fromJson) => then((response) => response.asJsonMap(fromJson));
}

extension ResponseConverter on XHttpResponse {
  Future<Uint8List> asBytes() => body.asBytes();

  Stream<List<int>> asStream() => body.asStream();

  Future<String> asText() => body.asBytes().then(encoding.decode);

  Future<T> asJson<T>(FromJson<T> fromJson) => asText().then((json) => fromJson(jsonDecode(json)));

  Future<T> asJsonObject<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJson(json));

  Future<List<T>> asJsonList<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonList(json, fromJson));

  Future<Map<String, T>> asJsonMap<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonMap(json, fromJson));
}
